"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TechnologyStackService = void 0;
var technology_1 = require("../model/technology");
var async_call_blocker_1 = require("../infrastructure/async-call-blocker");
var readJson = require("read-package-json");
var fs = require('fs');
var path = require("path");
var TechnologyStackService = /** @class */ (function () {
    function TechnologyStackService() {
        this.technologies = [
            libFor("@angular/core", "Angular Core"),
            libFor("@angular/material", "Angular Material"),
            libFor("@angular/forms", "Angular Forms"),
            libFor("@angular/router", "Angular Router"),
            libFor("rxjs", "RxJS"),
            libFor("@ngrx/effects", "NgRx Effects"),
            libFor("@ngrx/store", "NgRx Store"),
            libFor("@ngrx/router-store", "NgRx Router Store")
        ];
    }
    TechnologyStackService.prototype.scan = function () {
        var technologies = [];
        technologies.push.apply(technologies, this.scanDependencies());
        var technicalNames = new Set();
        technologies.forEach(function (tech) { return technicalNames.add(tech.technicalName); });
        // only add when not already added
        this.scanTransitiveDependencies()
            .filter(function (tech) { return !technicalNames.has(tech.technicalName); })
            .forEach(function (tech) { return technologies.push(tech); });
        technologies.push.apply(technologies, this.scanGitHubActionDependencies()); // no need to check whether already added
        return technologies;
    };
    TechnologyStackService.prototype.scanGitHubActionDependencies = function () {
        try {
            var filename = ".github";
            if (fs.existsSync(filename)) {
                return this.findGitHubDependencies(filename, []);
            }
            else {
                return [];
            }
        }
        catch (err) {
            console.error("Error while scanning github action dependencies: " + err);
            return [];
        }
    };
    TechnologyStackService.prototype.scanTransitiveDependencies = function () {
        var _this = this;
        try {
            var filename = "package-lock.json";
            if (fs.existsSync(filename)) {
                var packageLockJsonContent_1 = this.readFileContent(filename);
                if (packageLockJsonContent_1 == null || packageLockJsonContent_1.dependencies == null) {
                    return [];
                }
                return Object.keys(packageLockJsonContent_1.dependencies).map(function (key) {
                    var dependencyObject = packageLockJsonContent_1.dependencies[key];
                    var tech = __assign(__assign({}, (0, technology_1.createTechnology)()), { version: dependencyObject["version"], technicalName: key, transitive: true });
                    var lib = _this.technologies.find(function (tech) { return tech.technicalName === key; });
                    if (!!lib) {
                        tech.wellKnown = true;
                        tech.displayName = lib.displayName;
                    }
                    return tech;
                });
            }
            else {
                console.log('No package-lock.json, skip scanning of transitive dependencies');
                return [];
            }
        }
        catch (err) {
            console.error("Error while scanning transitive dependencies: " + err);
            return [];
        }
    };
    TechnologyStackService.prototype.scanDependencies = function () {
        var _this = this;
        var packageJsonContent = this.readPackageJson("package.json");
        if (packageJsonContent == null || packageJsonContent.dependencies == null) {
            return [];
        }
        return Object.keys(packageJsonContent.dependencies).map(function (key) {
            var version = packageJsonContent.dependencies[key];
            var tech = __assign(__assign({}, (0, technology_1.createTechnology)()), { version: version, technicalName: key });
            var lib = _this.technologies.find(function (tech) { return tech.technicalName === key; });
            if (!!lib) {
                tech.wellKnown = true;
                tech.displayName = lib.displayName;
            }
            return tech;
        });
    };
    TechnologyStackService.prototype.findGitHubDependencies = function (dirPath, arrayOfFiles) {
        var _this = this;
        var files = fs.readdirSync(dirPath);
        arrayOfFiles = arrayOfFiles || [];
        files.forEach(function (file) {
            if (fs.statSync(dirPath + "/" + file).isDirectory()) {
                arrayOfFiles = _this.findGitHubDependencies(dirPath + "/" + file, arrayOfFiles);
            }
            else {
                if (file.toUpperCase().endsWith('.YML')
                    || file.toUpperCase().endsWith('.YAML')) {
                    arrayOfFiles.push.apply(arrayOfFiles, _this.extractTechnologiesFromGitHubAction(dirPath + "/" + file));
                }
                // arrayOfFiles.push(path.join(__dirname, dirPath, "/", file))
            }
        });
        return arrayOfFiles;
    };
    TechnologyStackService.prototype.extractTechnologiesFromGitHubAction = function (file) {
        var yamlContent = fs.readFileSync(file, 'utf8').toString();
        var techs = [];
        yamlContent.split('\n')
            .map(function (line) { return line.replace('\r', '').replace('\t', ''); })
            .map(function (line) { return line.trim(); })
            .filter(function (line) { return !line.startsWith('#'); })
            .filter(function (line) { return line.startsWith('uses:') || line.startsWith('- uses:'); })
            .forEach(function (line) {
            var index = line.indexOf('uses:');
            var value = line.substring(index + 5).trim();
            var parts = value.split('@');
            techs.push(__assign(__assign({}, (0, technology_1.createTechnology)()), { technicalName: parts[0], version: (parts.length > 1) ? parts[1] : null }));
        });
        return techs;
    };
    TechnologyStackService.prototype.readFileContent = function (file) {
        // Directly using fs instead of readJson, because readJson would do unwanted extra Json parsing
        return JSON.parse(fs.readFileSync(file, 'utf8').toString());
    };
    TechnologyStackService.prototype.readPackageJson = function (file) {
        var fileContent = null;
        readJson(file, null, false, function (er, data) {
            if (er) {
                console.error("Error: There was an error reading " + file);
                return;
            }
            fileContent = data;
        });
        // Block until file is read
        while (!fileContent) {
            (0, async_call_blocker_1.blockExecution)();
        }
        return fileContent;
    };
    return TechnologyStackService;
}());
exports.TechnologyStackService = TechnologyStackService;
function libFor(technicalName, displayName) {
    return { displayName: displayName, technicalName: technicalName };
}
